/***********************************************************************/
/*                                                                     */
/*               Output Analysis Program for ETMSP (OAP)               */
/*               =======================================               */
/*                                                                     */
/*        copyright @(1993) Electric Power Research Institute          */
/*                                                                     */
/* EPRI reserves all rights in the software and associated             */
/* documentation as delivered. The software and its associated         */
/* documention or any portion thereof may not be reproduced in any form*/
/* whatsoever without the consent of EPRI, except as explicity provided*/
/* for by license. Licensee shall submit a detailed description to EPRI*/
/* of any changes or modifications to the software within two (2)      */
/* months after such changes or modifications are made. At EPRI's sole */
/* discretion, the software, as changed or modified, shall be given a  */
/* new designation sufficiently different from its then current        */
/* designation as to prevent mistake,confusion, or deception as between*/
/* the current software as changed or modified.                        */
/*                                                                     */
/* A license under EPRI's rights in the software can be obtained       */
/* directly from EPRI.                                                 */
/*                                                                     */
/* This program and supporting materials may be independently available*/
/* from Electric Power Software centre (EPSC) for an appropriate       */
/* distribution fee. The EPSC is a service of                          */
/*                                                                     */
/*                    Power Computing Company                          */
/*                    1930 Hi Line                                     */
/*                    Dallas, Tx 75207                                 */
/*                                                                     */
/* NEITHER EPRI, ANY MEMBER OF EPRI, NOR ANY PERSON OR  ORGANIZATION   */
/* ACTING ON BEHAVE OF THEM:                                           */
/*                                                                     */
/* 1. MAKE ANY WARRANTY OR REPRESENTATION WHATSOEVER, EXPRESS OR       */
/*    IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY OR FITNESS OR */
/*    ANY PURPOSE WITH RESPECT TO THE SOFTWARE OR ITS ASSOCIATED       */
/*    DOCUMENTATION; OR                                                */
/*                                                                     */
/* 2. ASSUMES ANY LIABILITY WHATSOEVER WITH RESPECT TO ANY USE OF THE  */
/*    SOFTWARE OR ITS ASSOCIATED DOCUMENTATION OR ANY PORTION THEREOF  */
/*    OR WITH RESPECT TO ANY DAMAGES WHICH MAY RESULT FROM SUCH USE.   */
/*                                                                     */
/* THIS NOTICE MAY NOT BE REMOVED FROM THE SOFTWARE BY ANY USER        */
/* THEREOF.                                                            */
/*                                                                     */
/***********************************************************************/

/***********************************************************
 *
 * File:          x3d.c
 * Description:   display OAP outputs in form of 3D plot
 *                on X11 Window system
 *
 ***********************************************************/

/*
 *    Include File
 */

/* if the code is compiled for IBM RS6000, take the
   following comments out */

#include "X3d_PS.h"
#include "hydro.h"
#include "epri.h"

#ifdef VMS
#include <Xlib.h>
#include <Xutil.h>
#include <Xos.h>
#else
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#endif

/*
 *    Function Declarations
 */

extern int textlen();
extern set_line_color();
/*
vector new_p_coordinate();
float surface_check();
void indexx();
plot3d();
draw_tick();
draw_axes();
Identity();
check_tickflag();
rotate();
matrix_op_mul();
rotate_around_z_axis();
rotate_around_y_axis();
wp_scale();
refresh_3d();
clear_3d_w_();
init_3d_w_();
transfor_m();
check_xyz();
event_3d_();
frame();
paint_3d_b();
init_3d_data_();
x2end_();
*/

/*
 *    Preprocessor
 */

#define UP 1
#define DOWN 2
#define RIGHT 3
#define LEFT 4
#define XOY 5
#define YOZ 6
#define ZOX 7
#define ANY 8
#define VXMIN 300
#define VXMAX 700
#define VYMIN 190
#define VYMAX 540
#define X_DIM 850
#define Y_DIM 700
#define X_ORG 10 
#define Y_ORG 10 

/*
 *    Global Variables
 */

Display		*display;
int     	scr;
int 		win_w, win_h;
GC 		gc,bgcr;
int 		background;
char    	*font_name;
XFontStruct 	*font_info;
Window 		wd, bt[9];
Pixmap 		hydro_pixmap,px_3d_map;
matrix  	trans_matrix;
matrix 		scal;
float 		xnew[1000],ynew[1000];
int 		lx, ly, lz;
int		depth;
char		*color_name[10];
unsigned long
		line_colors[10];
static int Xmin=200, Xmax=815, Ymin=115, Ymax=565;

/***********************************************************
 * obtain transformed point vector in new coordinate 
 */
vector new_p_coordinate (point,M)
vector point;
matrix M;
{
   float temp[4];
   vector new_p;
   int i;

   for (i=0; i<4; i++) 
      temp[i] = point.x *M[0][i]
          + point.y *M[1][i] + point.z *M[2][i] + M[3][i];
   new_p.x = temp[0]/temp[3];
   new_p.y = temp[1]/temp[3];
   new_p.z = temp[2]/temp[3];
   return(new_p);
}

/***********************************************************
 * check if the surface is visible from current view point
 */
float surface_check(x1,y1,x2,y2,x3,y3)
float x1,y1,x2,y2,x3,y3;
{
   float test;

   test=(x2-x1)*(y3-y2)-(y2-y1)*(x3-x2);
   return (test);
}

/***********************************************************
 * get the index of the surface in order of distance from
 * current view point
 */
void indexx(n,arrin,indx)
int n, indx[];
float arrin[];
{
   int l, j, ir, indxt, i;
   float q;
   float *bb;

   bb=arrin-1;
   for (j=1; j<=n; j++) indx[j]=j;
   if (n==1) return;
   l=(n>>1) +1;
   ir = n;
   for (;;)
   {
     if (l > 1)
            q=bb[(indxt = indx[--l])];
     else
     {
            q=bb[(indxt=indx[ir])];
            indx[ir]=indx[1];
            if (--ir == 1)
            {
              indx[1] = indxt;
              return;
            }
     }
     i=l;
     j=l << 1;
     while (j <= ir)
     {
            if (j < ir && bb[indx[j]] < bb[indx[j+1]]) j++;
            if (q < bb[indx[j]])
            {
              indx[i] = indx[j];
              j += (i=j);
            }
            else j =ir +1;
     }
     indx[i]=indxt;
   }
}

/***********************************************************
 * 3D plot
 */
plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
    yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc)
float xtem[], ytem[], ztem[];
float xaray[],yaray[];
float xs[],xd[],ys[],yd[];
int *idx1,*idx2;
int *maxpt,*max_p,*iflg; /* max_p number of plate */
int *icc, *ics;   /* icc number of cylinder */
		  /* ics position in the array */
{
   XPoint xpoint[8000],spoint[300];
   vector point, new_point, new_point1, new_point2, new_point3; 
   int i, j, id, ip, ix, iyz, is,mx1,ny1,mx2,ny2;
   int icy,iz,order[8001];
   int mtemp;
   float test;
   float zid[8000];
   double x21[4], y21[4], dd[4];

   /* frame, axes, ticks and labels */
   frame();
   draw_axes(1);

   if (*iflg==2) {
   /* plot realy margin */
  
   /* plot line bindings in 3d space */
     for (i=0; i<*max_p; i++)
     {
      point.x=xtem[0];point.y=xs[i];point.z=ys[i];
      new_point1 = new_p_coordinate(point,trans_matrix);
      point.x=xtem[*maxpt-1];point.y=xs[i];point.z=ys[i];
      new_point2 = new_p_coordinate(point,trans_matrix);
      point.x=xtem[*maxpt-1];point.y=xd[i];point.z=yd[i];
      new_point3 = new_p_coordinate(point,trans_matrix);
      point.x=xtem[i];point.y=xd[i];point.z=yd[i];
      new_point = new_p_coordinate(point,trans_matrix);
      spoint[0].x = new_point1.x;
      spoint[0].y = Y_DIM-new_point1.y;
      spoint[1].x = new_point2.x;
      spoint[1].y = Y_DIM-new_point2.y;
      spoint[2].x = new_point3.x;
      spoint[2].y = Y_DIM-new_point3.y;
      spoint[3].x = new_point.x;
      spoint[3].y = Y_DIM-new_point.y;
      spoint[4].x = new_point1.x;
      spoint[4].y = Y_DIM-new_point1.y;
      XDrawLines(display,px_3d_map,gc,spoint,5,CoordModeOrigin);
     }

     iz=0;
     for(icy=0; icy<*icc; icy++)
     {
     /* plot two end surfaces */
  	for (i=0;i<2;i++){       
     	   id = i*(*maxpt-1);
     	   ip=0;
     	   for (j=0 ; j<maxpt[1]; j++) {
      		point.x=xtem[id]; 
		point.y=xaray[ics[icy]+j];
      		point.z=yaray[ics[icy]+j];
      		new_point = new_p_coordinate(point,trans_matrix);
         	spoint[ip].x = new_point.x;
         	spoint[ip].y = Y_DIM-new_point.y;
         	ip=ip+1;
     	   }

	   /* test if it's visible */
           new_point1.x=(float)spoint[0].x; 
           new_point1.y=(float)(Y_DIM-spoint[0].y);
           new_point2.x=(float)spoint[10].x;
           new_point2.y=(float)(Y_DIM-spoint[10].y);
           new_point3.x=(float)spoint[20].x;
           new_point3.y=(float)(Y_DIM-spoint[20].y);
           if (i==0)
        	test=surface_check(new_point1.x,new_point1.y,
        		new_point2.x,new_point2.y,
			new_point3.x,new_point3.y);
           else
    		test=surface_check(new_point3.x,new_point3.y,
        		new_point2.x,new_point2.y,
			new_point1.x,new_point1.y);
           if (test>=0.) {
      		XSetLineAttributes(display,gc,1,LineSolid,
			CapButt,JoinBevel); 
      		XDrawLines(display,px_3d_map,gc,spoint,ip,
			CoordModeOrigin);
     	   }
           else {
        	XSetLineAttributes(display,gc,1,LineOnOffDash,
			CapButt,JoinBevel);
      		XDrawLines(display,px_3d_map,gc,spoint,ip,
			CoordModeOrigin);
           }
  	}
	XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);

     /* plot side surfaces */
	ix =10;  
	iyz = 25;
	is = ix * iyz;
	for(i=0; i<is; i++){
     		mx1=(int)((int)(i%ix)*(*maxpt-1.)/ix);
     		ny1=(int)(floor((float)(i/ix))*(maxpt[1]-1.)/iyz);
     		if ((int)(i%ix)==(ix-1))
       			mx2= *maxpt-1;
     		else
       			mx2=(int)((int)(i%ix+1)*(*maxpt-1.)/ix);
    		ny2=(int)(floor
			    ((float)(i/ix+1.))*(maxpt[1]-1.)/iyz);
    		point.x=xtem[mx1];
		point.y=xaray[ics[icy]+ny1];
    		point.z=yaray[ics[icy]+ny1];
    		new_point1= new_p_coordinate(point,trans_matrix);
    		xpoint[iz*5].x = new_point1.x;
    		xpoint[iz*5].y = Y_DIM-new_point1.y;   

    		point.x=xtem[mx2];
		point.y=xaray[ics[icy]+ny1];
		point.z=yaray[ics[icy]+ny1];
    		new_point2= new_p_coordinate(point,trans_matrix);
    		xpoint[iz*5+1].x = new_point2.x;
    		xpoint[iz*5+1].y = Y_DIM-new_point2.y;

    		point.x=xtem[mx2];
		point.y=xaray[ics[icy]+ny2];
		point.z=yaray[ics[icy]+ny2];
    		new_point3= new_p_coordinate(point,trans_matrix);
    		xpoint[iz*5+2].x = new_point3.x;
    		xpoint[iz*5+2].y = Y_DIM-new_point3.y;

    		test=surface_check(new_point1.x,new_point1.y,
      			new_point2.x,new_point2.y,
      			new_point3.x,new_point3.y);
    		if(test>=0.)
    		{
     			point.x=xtem[mx1];
			point.y=xaray[ics[icy]+ny2];
			point.z=yaray[ics[icy]+ny2];
     			new_point = new_p_coordinate(point,trans_matrix);
     			xpoint[iz*5+3].x = new_point.x;
     			xpoint[iz*5+3].y = Y_DIM-new_point.y;
     			xpoint[iz*5+4].x = new_point1.x;
     			xpoint[iz*5+4].y = Y_DIM-new_point1.y;
     			zid[iz]= -(new_point1.z+new_point3.z)/2.;
     			iz++;
    		}
	} /* end of for(i) */
     } /* end of for(icy) */
   
     indexx(iz,zid,order);
     for (i=0; i<iz; i++) 
     {
        ip=order[i+1]-1;
        spoint[0].x=xpoint[ip*5].x;
        spoint[0].y=xpoint[ip*5].y;
        spoint[1].x=xpoint[ip*5+1].x;
        spoint[1].y=xpoint[ip*5+1].y;
        spoint[2].x=xpoint[ip*5+2].x;
        spoint[2].y=xpoint[ip*5+2].y;
        spoint[3].x=xpoint[ip*5+3].x;
        spoint[3].y=xpoint[ip*5+3].y;
        spoint[4].x=spoint[0].x;
        spoint[4].y=spoint[0].y;
        XSetForeground(display, gc,  WhitePixel(display,scr));
        XFillPolygon(display, px_3d_map, gc, spoint, 5,
	       Convex,CoordModeOrigin);
        XSetForeground(display, gc, BlackPixel(display,scr));
        XDrawLines(display,px_3d_map,gc,spoint,5,CoordModeOrigin);
     }
     
   }  /* end of if(iflg) */

   /* plot curve */
   if (*iflg==4) 
   {

    /* plot surface for 3D curves */
    j = (theta*phi>0) ? *max_p-1 : 1;
    while(j > 0 && j < *max_p) {
        ny2=0;
        test = phi + 1.570796326795;
        if(test>3.14159265359) test -= 6.28318539718;
        id = (theta*test>0) ? 50 : 1;
        while(id > 0 && id < 51) {
            ip=0;
            ix=((id-1)*(idx2[j-1]-idx1[j-1])+1)/50;
            mx2=(id*(idx2[j-1]-idx1[j-1])+1)/50;
            while(ix<=mx2){
                point.x = xaray[idx1[j-1]-1+ix+(j-1)*1000];
                point.z = yaray[idx1[j-1]-1+ix+(j-1)*1000];
                point.y = ztem[j-1];
                new_point = new_p_coordinate(point,trans_matrix);
                xpoint[ip].x = new_point.x;
                xpoint[ip++].y = Y_DIM-new_point.y;
                ix++;
            }
            ny1=((id-1)*(idx2[j]-idx1[j])+1)/50;
            iz=(id*(idx2[j]-idx1[j])+1)/50;
            while(iz>=ny1){
                point.x = xaray[idx1[j]-1+iz+j*1000];
                point.z = yaray[idx1[j]-1+iz+j*1000];
                point.y = ztem[j];
                new_point = new_p_coordinate(point,trans_matrix);
                xpoint[ip].x = new_point.x;
                xpoint[ip++].y = Y_DIM-new_point.y;
                iz--;
            }
            xpoint[ip].x=xpoint[0].x;
            xpoint[ip++].y=xpoint[0].y;
            XSetForeground(display, gc, WhitePixel(display, scr));
            XFillPolygon(display, px_3d_map, gc, xpoint, ip,
                Complex, CoordModeOrigin);
            XSetForeground(display, gc, BlackPixel(display, scr));
            XDrawLines(display, px_3d_map, gc, xpoint, ip, CoordModeOrigin);
            (theta*test>0) ? id-- : id++;
        }
        theta*phi>0 ? j-- : j++;
    }
   }
   else
   {
     mtemp=idx2[0]-idx1[0]+1;
     if (!(depth==1))
        set_line_color(1);
     for (j=0 ; j<mtemp;j++) {
        point.x=xtem[idx1[0]-1+j];
        point.y=ytem[idx1[0]-1+j];
        point.z=ztem[idx1[0]-1+j];
        new_point = new_p_coordinate(point,trans_matrix);
        xnew[j]=new_point.x; ynew[j]= Y_DIM-new_point.y;
        xpoint[j].x = new_point.x;
        xpoint[j].y = Y_DIM-new_point.y;
     }
     XDrawLines(display,px_3d_map,gc,xpoint,mtemp,CoordModeOrigin);
     set_line_color(0);
   }

   draw_axes(0);
}

/***********************************************************
 * draw ticks
 */
draw_tick(id, vst, ved, rmin, rmax, ratio,ddx,ddy,ddz)
vector vst, ved;
int id;
float rmin,rmax,ratio,ddx,ddy,ddz;
{
   vector tem1,tem2,newt1,newt2;
   float dd,dc,t_label;
   char text[30];
   int len,xpos,ypos;
   int i;
   XPoint  xpoint[2];

   dc=(rmax-rmin)/5.;
   switch (id)
   {
     case 0:  /* x axis ticks and annotations */
      dd=(ved.x-vst.x)/5.;
      for (i=0; i<=5; i++) {
        tem1.x = tem2.x = vst.x +i*dd;
        tem1.y = vst.y-ddy; tem2.y = vst.y +ddy;
        tem1.z = vst.z-ddz; tem2.z = vst.z +ddz;;
        newt1 = new_p_coordinate(tem1,trans_matrix);
        newt2 = new_p_coordinate(tem2,trans_matrix);
        xpoint[0].x=newt1.x;
        xpoint[0].y=Y_DIM-newt1.y;
        xpoint[1].x=newt2.x;
        xpoint[1].y=Y_DIM-newt2.y;
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);
        t_label=rmin+i*dc;
        tem1.x = vst.x+i*dd;
        tem1.y = vst.y + ratio*ddy; tem1.z = vst.z + ratio*ddz;
        newt1 = new_p_coordinate(tem1,trans_matrix);
        sprintf(text,"%4.2f",t_label);
        len=strlen(text);
        xpos=(int)newt1.x;
        ypos=(int)(Y_DIM-newt1.y);
        XDrawString(display,px_3d_map,gc,xpos,ypos,text,len);
      }

      /* x label */
      tem1.x = (vst.x+ved.x)/2.;
      tem1.y = (vst.y+ved.y)/2.+ 2.*ratio*ddy;
      tem1.z= (vst.z+ved.z)/2.+ 2.*ratio*ddz;
      newt1 = new_p_coordinate(tem1,trans_matrix);
      xpos=(int)newt1.x;
      ypos=(int)(Y_DIM-newt1.y);
      XDrawString(display,px_3d_map,gc,xpos,ypos, "X",1);
      break;

     case 1: /* y axis ticks and annotations */
      dd=(ved.y-vst.y)/5.;
      for (i=0; i<=5; i++) {
        tem1.y = tem2.y = vst.y+i*dd;
        tem1.x = vst.x - ddx; tem2.x = vst.x + ddx;
        tem1.z = vst.z -ddz; tem2.z  = vst.z + ddz;
        newt1 = new_p_coordinate(tem1,trans_matrix);
        newt2 = new_p_coordinate(tem2,trans_matrix);
        xpoint[0].x=newt1.x;
        xpoint[0].y=Y_DIM-newt1.y;
        xpoint[1].x=newt2.x;
        xpoint[1].y=Y_DIM-newt2.y;
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);
        t_label=rmin+i*dc;
        tem1.y = vst.y+i*dd;
        tem1.x = vst.x + ratio*ddx; tem1.z = vst.z+ratio*ddz;
        newt1 = new_p_coordinate(tem1,trans_matrix);
        sprintf(text,"%4.2f",t_label);
        len=strlen(text);
        xpos=(int)newt1.x;
        ypos=(int)(Y_DIM-newt1.y);
        XDrawString(display,px_3d_map,gc,xpos,ypos,text,len);
      }

      /* y label */
      tem1.y= (vst.y+ved.y)/2.;
      tem1.x =(vst.x+ved.x)/2.+ 1.5*ratio*ddx;
      tem1.z=(vst.z+ved.z)/2. + 1.5*ratio*ddz;
      newt1 = new_p_coordinate(tem1,trans_matrix);
      xpos=(int)newt1.x;
      ypos=(int)(Y_DIM-newt1.y);
      XDrawString(display,px_3d_map,gc,xpos,ypos, "Y",1);
      break;

     case 2: /* z axis ticks and annotations */
      dd=(ved.z-vst.z)/5.;
      for (i=0; i<=5; i++) {
        tem1.z = tem2.z = vst.z+i*dd;
        tem1.y = vst.y -ddy; tem2.y = vst.y +ddy;
        tem1.x = vst.x-ddx; tem2.x = vst.x+ddx;
        newt1 = new_p_coordinate(tem1,trans_matrix);
        newt2 = new_p_coordinate(tem2,trans_matrix);
        xpoint[0].x=newt1.x;
        xpoint[0].y=Y_DIM-newt1.y;
        xpoint[1].x=newt2.x;
        xpoint[1].y=Y_DIM-newt2.y;
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);
        t_label=rmin+i*dc;
        tem1.z = vst.z+i*dd;
        tem1.x = vst.x +ratio*ddx;
        tem1.y = vst.y +ratio*ddy;
        newt1 = new_p_coordinate(tem1,trans_matrix);
        sprintf(text,"%4.2f",t_label);
        len=strlen(text);
        xpos=(int)newt1.x;
        ypos=(int)(Y_DIM-newt1.y);
        XDrawString(display,px_3d_map,gc,xpos,ypos,text,len);
      }

      /* z label */
      tem1.z=(vst.z+ved.z)/2.;
      tem1.x = (vst.x+ved.x)/2.+ 1.5*ratio*ddx;
      tem1.y=(vst.y+ved.y)/2.+ 1.5*ratio*ddy;
      newt1 = new_p_coordinate(tem1,trans_matrix);
      xpos=(int)newt1.x;
      ypos=(int)(Y_DIM-newt1.y);
      XDrawString(display,px_3d_map,gc,xpos,ypos, "Z",1);
      break;
   }
}

/***********************************************************
 * choose which axis ticks and labels should be put on;
 * call draw_tick to draw ticks and labels
 */
draw_axes(j)
int j;
{
   XPoint  xpoint[5];
   vector new_oo,new_xo,new_yo,new_zo,new_xoy,new_yoz,new_zox,new_xyz;
   float test,t[4];
   int i, id,xy_tick_index;

   /* get window coordinates of vector oo, xo, yo, zo,
      xoy, yoz, zox, xyz */
   new_oo=new_p_coordinate(oo,trans_matrix);
   new_xo=new_p_coordinate(xo,trans_matrix);
   new_yo=new_p_coordinate(yo,trans_matrix);
   new_zo=new_p_coordinate(zo,trans_matrix);
   new_xoy=new_p_coordinate(xoy,trans_matrix);
   new_yoz=new_p_coordinate(yoz,trans_matrix);
   new_zox=new_p_coordinate(zox,trans_matrix);
   new_xyz=new_p_coordinate(xyz,trans_matrix);

    test = phi + 1.570796326795;
    if(test>3.14159265359) test -= 6.28318539718;

    /* draw cube */
    XSetLineAttributes(display,gc,1,LineOnOffDash,CapButt,JoinBevel);

    xpoint[0].x=new_xyz.x; xpoint[0].y=Y_DIM-new_xyz.y;
    xpoint[1].x=new_xoy.x; xpoint[1].y=Y_DIM-new_xoy.y;
    if (j || (theta*test<0 && theta*phi<0))
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[0].x=new_yo.x;  xpoint[0].y=Y_DIM-new_yo.y;
    if (j || theta*phi<0)
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[1].x=new_yoz.x; xpoint[1].y=Y_DIM-new_yoz.y;
    if (j || (theta*test>0 && theta*phi<0))
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[0].x=new_xyz.x; xpoint[0].y=Y_DIM-new_xyz.y;
    if (j || theta*phi<0)
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[0].x=new_zo.x;  xpoint[0].y=Y_DIM-new_zo.y;
    if (j || theta*test>0)
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[1].x=new_zox.x; xpoint[1].y=Y_DIM-new_zox.y;
    if (j || theta*phi>0)
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[0].x=new_xyz.x; xpoint[0].y=Y_DIM-new_xyz.y;
    if (j || theta*test<0)
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[0].x=new_xo.x;  xpoint[0].y=Y_DIM-new_xo.y;
    if (j || (theta*test<0 && theta*phi>0))
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[1].x=new_xoy.x; xpoint[1].y=Y_DIM-new_xoy.y;
    if (j || theta*test<0)
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[1].x=new_oo.x; xpoint[1].y=Y_DIM-new_oo.y;
    if (j || theta*phi>0)
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[0].x=new_yo.x; xpoint[0].y=Y_DIM-new_yo.y;
    if (j || theta*test>0)
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

    xpoint[0].x=new_zo.x; xpoint[0].y=Y_DIM-new_zo.y;
    if (j || (theta*test>0 && theta*phi>0))
        XDrawLines(display,px_3d_map,gc,xpoint,2,CoordModeOrigin);

   XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);

    if(!j) return;

   /* draw ticks, annotations ans labels */
   if(tickflag==8) {
        t[0]=new_oo.x+new_zo.x;
        t[1]=new_xoy.x+new_xyz.x;
        t[2]=new_xo.x+new_zox.x;
        t[3]=new_yo.x+new_yoz.x;
        test=100000.;
        for (i=0; i<4; i++){
           if (t[i]<test) {
                test=t[i];
                id=i; }
        }
        t[0]=0.02*(xmax-xmin);
        t[1]=0.02*(ymax-ymin);
        if (id==0) {
	  draw_tick(2, oo, zo, zmin, zmax, -6.,t[0], t[1], 0.);
	  if (new_zo.y>new_oo.y) 
	    if (new_yo.y>new_xo.y)
		xy_tick_index=0;
	    else
		xy_tick_index=2;
	  else 
	    if (new_yo.y<new_xo.y)
		xy_tick_index=1;
	    else
		xy_tick_index=3;
	}
        if (id==1) {
	  draw_tick(2, xoy, xyz, zmin, zmax,6.,t[0], t[1], 0.);
	  if (new_xyz.y>new_xoy.y) 
	    if (new_yo.y<new_xo.y)
		xy_tick_index=2;
	    else
		xy_tick_index=0;
	  else 
	    if (new_yo.y>new_xo.y)
                xy_tick_index=3;
	    else
		xy_tick_index=1;
        }
        if (id==2) {
	  draw_tick(2, xo, zox, zmin, zmax, 6.,t[0], -t[1], 0.);
	  if (new_zox.y>new_xo.y) 
	    if (new_xoy.y < new_oo.y)
		xy_tick_index=4;
	    else
		xy_tick_index=6;
	  else 
	    if (new_xoy.y > new_oo.y)
                xy_tick_index=5;
	    else
		xy_tick_index=7;
	}
        if (id==3) {
	  draw_tick(2, yo, yoz, zmin, zmax, 6.,-t[0], t[1] , 0.);
	  if (new_yoz.y>new_yo.y) 
	    if (new_xoy.y > new_oo.y)
		xy_tick_index=6;
	    else
		xy_tick_index=4;
	  else 
	    if (new_xoy.y < new_oo.y)
                xy_tick_index=7;
	    else
		xy_tick_index=5;
	}
     
	switch(xy_tick_index) {
	 case 0:
	   draw_tick(0, oo, xo, xmin, xmax, -6., 0., .02*(ymax-ymin), 0.);
	   draw_tick(1, xo, xoy, ymin, ymax, 6.,.02*(xmax-xmin), 0.,0.); 
	   break;
	 case 1:
	   draw_tick(0, yoz, xyz,xmin, xmax, 6.,0.,.02*(ymax-ymin), 0.);
	   draw_tick(1, zo, yoz,  ymin, ymax,-6.,.02*(xmax-xmin), 0.,0.);
	   break;
	 case 2:
	   draw_tick(0, yo, xoy, xmin, xmax, 6., 0., .02*(ymax-ymin), 0.);
	   draw_tick(1, oo, yo, ymin, ymax, -6.,.02*(xmax-xmin), 0., 0.);
	   break;
	 case 3:
	   draw_tick(0, zo, zox,xmin, xmax, -6., 0., .02*(ymax-ymin), 0.);
	   draw_tick(1, zox, xyz, ymin, ymax, 6.,.02*(xmax-xmin), 0., 0.);
	   break;
	 case 4:
	   draw_tick(0, yo, xoy, xmin, xmax, 6., 0., .02*(ymax-ymin), 0.);
	   draw_tick(1, xo, xoy, ymin, ymax, 6.,.02*(xmax-xmin), 0.,0.);
	   break;
	 case 5:
	   draw_tick(0, zo, zox,  xmin, xmax, -6., 0., .02*(ymax-ymin), 0.);
	   draw_tick(1, zo, yoz,  ymin, ymax,-6.,.02*(xmax-xmin), 0.,0.);
	   break;
	 case 6:
	   draw_tick(0, oo, xo, xmin, xmax, -6., 0., .02*(ymax-ymin), 0.);
           draw_tick(1, oo, yo, ymin, ymax, -6.,.02*(xmax-xmin), 0., 0.);
	   break;
	 case 7:
	   draw_tick(0, yoz, xyz, xmin, xmax, 6., 0., .02*(ymax-ymin), 0.);
	   draw_tick(1, zox, xyz, ymin, ymax, 6.,.02*(xmax-xmin), 0., 0.);
	   break;
        }
   }

   /* draw ticks, annotations ans labels  for XY plot  */
   if(tickflag==5){
           draw_tick(0, oo, xo, xmin, xmax, -3., 0., .02*(ymax-ymin), 0.);
	   draw_tick(1, oo, yo, ymin, ymax, -6.,.02*(xmax-xmin), 0., 0.);
   }

   /* draw ticks, annotations ans labels  for YZ plot  */
   if(tickflag==6){
           draw_tick(1, oo, yo, ymin, ymax, -3.,0., 0.,.02*(zmax-zmin));
           draw_tick(2, oo, zo, zmin, zmax, -6.,0.,.02*(ymax-ymin), 0.);
     }

   /* draw ticks, annotations ans labels  for ZX plot  */
   if(tickflag==7){
           draw_tick(0, oo, xo, xmin, xmax, -3., 0.,0., .02*(zmax-zmin));
           draw_tick(2, oo, zo, zmin, zmax, -6.,.02*(xmax-xmin),0., 0.);
   }
}

/***********************************************************
 * set identity matrix
 */
Identity(M)
matrix M;
{
   int i,j;

   for (i=0; i<4; i++)
      for (j=0; j<4; j++)
         if (i==j)
            M[i][j] = 1.0;
         else
            M[i][j] = 0.0;
}

/***********************************************************
 * check tickflag
 *	5. XY plane
 *	6. YZ plane
 *	7. XZ plane
 *	8. other
 */
check_tickflag()
{
   tickflag=ANY;
   if(theta==0. && phi == -1.57078802585601807) 
	tickflag=XOY;
   if(phi==0. && theta==1.57078802585601807) 
	tickflag=YOZ;
   if(phi == -1.57078802585601807 &&
              theta==1.57078802585601807) 
	tickflag=ZOX;
}

/***********************************************************
 * find transformation matrix for current view angle
 */
rotate(id)
int id;
{
   float temp;
   clear_3d_w_();
   switch(id) {
 	case UP: 
		  theta=theta+5.*D_RAD;
                  if(theta>3.14159265359) theta -= 6.28318539718;
		  check_tickflag();
                  transfor_m(); 
                  break;
	case DOWN:
                  theta=theta-5.*D_RAD;
                  if(theta<-3.14159265359) theta += 6.28318539718;
		  check_tickflag();
                  transfor_m();
                  break;
	case RIGHT:
		  phi=phi+5.*D_RAD;
                  if(phi>3.14159265359) phi -= 6.28318539718;
		  check_tickflag();
                  transfor_m();
                  break;
	case LEFT:
                  phi=phi-5.*D_RAD;
                  if(phi<-3.14159265359) phi += 6.28318539718;
		  check_tickflag();
                  transfor_m();
                  break;
	case XOY: phi= -90.*D_RAD; theta=0.;
		  check_tickflag();
		  transfor_m();
                  break;
	case YOZ: phi=0.; theta=90.*D_RAD;
		  check_tickflag();
                  transfor_m();
                  break;
	case ZOX: phi= -90.*D_RAD; theta=90.*D_RAD;
		  check_tickflag();
                  transfor_m();
                  break;
	case ANY: 
        	  printf("input phi\n");
        	  scanf("%f",&temp);
       		  phi=temp*D_RAD;
        	  printf("input theta\n");
        	  scanf("%f",&temp);
        	  theta=temp*D_RAD;
		  check_tickflag();
                  transfor_m();
                  break;
	default: break;
   }
}

/***********************************************************
 * matrix multiplication
 */
matrix_op_mul(a,b,c)
matrix a,b,c;
{
  int i,j;
  for (i=0; i<4; i++)
   for (j=0; j<4; j++)
   {
      a[i][j] = b[i][0]*c[0][j]+b[i][1]*c[1][j]
	    +b[i][2]*c[2][j]+b[i][3]*c[3][j];
   }
}

/***********************************************************
 * get transformation matrix for the object rotating around 
 * z axis by an angle
 */
rotate_around_z_axis(angle,M)
float angle;
matrix M;
{
   Identity(M);
   M[0][0] =  cos(angle);
   M[0][1] = -sin(angle);
   M[1][0] =  sin(angle);
   M[1][1] =  cos(angle);
}

/***********************************************************
 * get transformation matrix for the object rotating around
 * x axis by an angle
 */
/* rotate_around_x_axis(angle,M)
float angle;
matrix M;
{
   Identity(M);
   M[1][1] =  cos(angle);
   M[1][2] = -sin(angle);
   M[2][1] =  sin(angle);
   M[2][2] =  cos(angle);
} */

/***********************************************************
 * get transformation matrix for the object rotating around
 * y axis by an angle
 */
rotate_around_y_axis(angle,M)
float angle;
matrix M;
{
   Identity(M);
   M[0][0] =  cos(angle);
   M[0][2] =  sin(angle);
   M[2][0] = -sin(angle);
   M[2][2] =  cos(angle);
}

/***********************************************************
 * get translation (scaling) matrix
 */
wp_scale(M,w_scal,xm,xa,ym,ya)
matrix M,w_scal; 
int xm,xa,ym,ya;
{
  dimension w, v;
  vector point;
  int i,j,k;
  float dx,dy;

   Identity(w_scal);
   v.left=xm; v.right=xa; v.bottom=ym; v.top=ya;
   w.left=1.0e10; w.right= -1.0e10; w.bottom=1.0e10; w.top= -1.0e10;
   for (i=0; i<2; i++)
   for (j=0; j<2; j++)
   for (k=0; k<2; k++) {
      point.x = xmin + i*(xmax-xmin);
      point.y = ymin + j*(ymax-ymin);
      point.z = zmin + k*(zmax-zmin);
      point = new_p_coordinate(point,M);
      if (point.x < w.left) w.left = point.x;
      if (point.x > w.right) w.right = point.x;
      if (point.y < w.bottom) w.bottom = point.y;
      if (point.y > w.top) w.top = point.y;
   }

   dx = w.right - w.left;
   dy = w.top - w.bottom;
   if (dx > dy) w.top = w.bottom + dx;
   if (dy > dx){
      w.left = w.left - 0.5*(dy-dx);
      w.right = w.right + 0.5*(dy-dx);
   }
   w_scal[0][0] = (v.right - v.left)/(w.right-w.left);
   w_scal[1][1] = (v.top - v.bottom)/(w.top-w.bottom);
   w_scal[3][0] = v.left - w_scal[0][0]*w.left;
   w_scal[3][1] = v.bottom - w_scal[1][1]*w.bottom;
}

/***********************************************************
 * refresh window
 */
refresh_3d()
{
   XCopyArea(display, px_3d_map, wd, gc,
           0, 0, win_w, win_h, 0, 0);
}

/***********************************************************
 * clear window
 */
clear_3d_w_()
{
   XFlush(display);
   XSetForeground(display, gc,  WhitePixel(display,scr));
   XFillRectangle(display,  px_3d_map, gc, 0, 0, win_w, win_h);
   XSetForeground(display, gc, BlackPixel(display,scr));
}

/***********************************************************
 * create top-level X window of the application
 * create button subwindows
 * set basic window parameters and hints
 */
init_3d_w_()
{
   XGCValues  gcvalues;
   char default_geometry[100];
   int  x, y;
   int i;
   char *display_name = NULL;
   char *geometry;
   char *wd_name = "ITOAP"; 
   char *icon_name = "OAP";
   XSizeHints  size_hints;
   XSetWindowAttributes attrib;
   int pane_height, menu_width, menu_height,offset_x,offset_y;
   int foreground;
   int white_bg;
   Colormap col_map;
   XColor scdef;

   attrib.backing_store = WhenMapped;

   /* Open display first, and get screen info */
   if (!(display = XOpenDisplay(display_name))) {
      fprintf(stderr,"Could not open display %s\n",
              display_name);
      exit(1);
   }
   scr = DefaultScreen(display);
   sprintf(default_geometry,"=%dx%d+%d+%d",X_DIM,Y_DIM,X_ORG,Y_ORG);

   foreground = BlackPixel(display,scr);
   white_bg = WhitePixel(display,scr);
   attrib.background_pixel = white_bg;

   depth = DisplayPlanes(display,scr);
   col_map = DefaultColormap(display,scr);
   if (depth == 1)
   {
      for (i=0; i<10; i++)
      {
        line_colors[i]= foreground;
      }
      background = white_bg;
   }
   else
   {
      color_name[0] = "black";
      color_name[1] = "red";
      color_name[2] = "green3";
      color_name[3] = "magenta1";
      color_name[4] = "purple3";
      color_name[5] = "chocolate1";
      /* color_name[6] = "turquoise1"; */
      color_name[6] = "dark olive green";
      color_name[7] = "spring green";
      color_name[8] = "LightSkyBlue1";
      color_name[9] = "cyan";
      for (i=0; i<10; i++)
      {
        if (XAllocNamedColor(display, col_map,
            color_name[i], &scdef, &scdef))
          line_colors[i]= scdef.pixel;
        else {
          fprintf(stderr,"Can't allocate color %s\n",
           color_name[i]);
          exit(1);
        }
      }
      background = line_colors[9];
   }

   /* Initialize the window */
   geometry="=+5+5";                  
   if (!XGeometry(display,scr,geometry,default_geometry,3,
                 0,0,0,0,&x,&y,&win_w,&win_h)) {
      x = X_ORG;
      y = Y_ORG;
      win_w = X_DIM;
      win_h = Y_DIM;
   }   
   /* create the window */
   wd =XCreateWindow(display,RootWindow(display,scr),
          x, y, win_w, win_h,3,
          DefaultDepth(display,scr), 
	  InputOutput,CopyFromParent,
          CWBackingStore, &attrib);

   /* create button subwindows */
   offset_x=60; offset_y=100;
   menu_height=320;
   menu_width=100;
   pane_height = (int)(menu_height/8);
   for  (i=0; i<9; i++) {
      bt[i]  = XCreateWindow(display,wd,
             offset_x, offset_y+pane_height*i,
	     menu_width,pane_height,2,
             DefaultDepth(display,scr), 
   	     InputOutput,CopyFromParent,
             CWBackingStore, &attrib);
      XSelectInput(display,bt[i], ButtonPressMask 
	     |ButtonReleaseMask | ExposureMask);
   }

   /* Initialize size hint property for window manager */
   size_hints.flags      = PPosition | PSize | PMinSize;
   size_hints.x          = x;
   size_hints.y          = y;
   size_hints.width      = win_w;
   size_hints.height     = win_h;
   size_hints.min_width  = X_DIM;
   size_hints.min_height = Y_DIM;

   /* creates a pixmap of "Ontario Hydro" as logo and
      icon pixmap */
#ifdef _LOGO_OH
   hydro_pixmap = XCreatePixmapFromBitmapData(display, wd, hydro_bits,
                       hydro_width, hydro_height,foreground,
                       white_bg,DefaultDepth(display, scr));
#endif
#ifdef _LOGO_EPRI 
   hydro_pixmap = XCreatePixmapFromBitmapData(display, wd, epri_bits,
                       epri_width, epri_height,foreground,
		       white_bg,DefaultDepth(display, scr));
#endif

   /* set standard properties of the window */
   XSetStandardProperties(display,wd, wd_name, icon_name,
                       hydro_pixmap, NULL, 0, &size_hints);

   /* select the event type to be sent to the top-level window */
   XSelectInput(display,wd, ExposureMask |
        	       ButtonPressMask | FocusChangeMask | 
	               KeyPressMask | StructureNotifyMask );

   /* Create GC for windows */
   gc = XCreateGC(display, wd, 0, &gcvalues);
   XSetForeground(display, gc, foreground);
   bgcr  = XCreateGC(display, wd, 0, NULL);
   XSetForeground(display, bgcr, foreground);

   /* Find a font and load it */
   font_name = startup_.defont;
   if (!(font_info = XLoadQueryFont(display,font_name))){
      fprintf(stderr,"Could not open font %s\n",font_name);
      exit(1);
   }
   XSetFont(display, gc, font_info->fid);

   /* creates a pixmap for 3D plot drawing area */
   px_3d_map = XCreatePixmap(display, wd, win_w, win_h,
           	       DefaultDepth(display, scr));

   /* Display the window */
   XMapSubwindows(display,wd);
   XMapWindow(display, wd);
}

/***********************************************************
 * calculate the transformation matrix for window coordinate
 */
transfor_m()
{
   char phi_label[15],theta_label[15];
   int text_len;
   matrix mz,my,mr,zup,w_scal;

   XDrawString(display,px_3d_map,gc,60,500,"View Angle(theta)",17);
   sprintf(theta_label,"%6.1f",theta*RAD_D);
   text_len   = strlen(theta_label);
   XDrawString(display,px_3d_map,gc,60,525,theta_label,text_len);
   XDrawString(display,px_3d_map,gc,60,550,"View angle(phi)",15);
   sprintf(phi_label,  "%6.1f", phi*RAD_D);
   text_len   = strlen(phi_label);
   XDrawString(display,px_3d_map,gc,60,575,phi_label,text_len);
   rotate_around_z_axis(phi,mz);
   rotate_around_y_axis(theta, my);
   rotate_around_z_axis(90.*D_RAD,zup);
   matrix_op_mul(mr,mz,my);
   matrix_op_mul(mz,mr,zup); 
   matrix_op_mul(Mr,scal,mz);
   wp_scale(Mr,w_scal,VXMIN,VXMAX,VYMIN,VYMAX);
   matrix_op_mul(trans_matrix,Mr,w_scal);
}

/***********************************************************
 * find the x,y,z value of a point in the curve 
 */
check_xyz(maxpt,xe,ye,idd)
int *maxpt,*idd;
int xe, ye;
{
   float temx, temy, temp, delta;
   int j;

   delta=100000.;
   for (j=0 ; j<*maxpt;j++) {
     temx=xnew[j]-xe; 
     temx=temx*temx;
     temy=ynew[j]-ye;
     temy=temy*temy;
     temp=sqrt(temx+temy);
     if (temp<delta) {
      delta=temp;
      *idd=j; 
     }
   }
}

/***********************************************************
 * window event handling
 */
event_3d_(idx1,idx2,maxpt,xtem,ytem,ztem,xaray,yaray,
          max_p, xs, ys, xd, yd, iflg, ics, icc)
float xtem[], ytem[], ztem[];
float xaray[],yaray[];
float xs[], ys[], xd[], yd[];
int *maxpt,*max_p,*iflg;
int *icc, *ics;
int idx1[],idx2[];
{
   XComposeStatus compose;
   char     buffer[1];
   int      bufsize = 1;
   int i;
   XSizeHints  size_hints;
   XEvent   event;
   int id;
   int win_x, win_y,x,y, xe, ye;
   unsigned int button;
   char labl[30];
   int text_len;

   phi= -120.*D_RAD;
   theta=120.*D_RAD;
   transfor_m();
   tickflag=ANY;
   plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
		yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
   refresh_3d();
   paint_3d_b();

   /* loop to get events on the main window and subwindows */
   while(1) {
    XNextEvent(display, &event);
    switch (event.type) {

   /* if expose, refresh the plot and paint button */
      case Expose:
         /* get rid of all other Expose events on the queue */
         while (XCheckTypedEvent(display, Expose, &event));
         transfor_m();
	 tickflag=ANY;
         plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
        	yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
         refresh_3d();
	 paint_3d_b();
         break;

      case ButtonPress:
   /* if ButtonPress occurs on the plot area of main window
      and iflg=2, print the x,y,z value of the point on 
      the screen */
         if (event.xbutton.window == wd && *iflg==2 ) {
            xe=event.xbutton.x;
            ye=event.xbutton.y;
            if(xe>250 && xe<750 && ye>100 && ye<600)
            {
                clear_3d_w_();
  		XDrawString(display,px_3d_map,gc,60,
			500,"View Angle(theta)",17);
  		sprintf(labl,"%6.1f",theta*RAD_D);
  		text_len   = strlen(labl);
  		XDrawString(display,px_3d_map,gc,60,
			525,labl,text_len);
  		XDrawString(display,px_3d_map,gc,60,
			550,"View angle(phi)",15);
  		sprintf(labl,  "%6.1f", phi*RAD_D);
  		text_len   = strlen(labl);
  		XDrawString(display,px_3d_map,gc,60,
			575,labl,text_len);
  		check_tickflag();
          	plot3d(idx1,idx2,xtem,ytem,ztem,xaray,yaray,
			maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
          	check_xyz(maxpt,xe,ye,&id);
	  	sprintf(labl,"X(Time in Second):%8.4f",xtem[id]);
  	  	text_len   = strlen(labl);
  	  	XDrawString(display,px_3d_map,gc,550,
			620,labl,text_len);
  	  	sprintf(labl,"Y(Impedance R):%8.4f", ytem[id]);
  	  	text_len   = strlen(labl);
  	  	XDrawString(display,px_3d_map,gc,550,
			640,labl,text_len);
       	  	sprintf(labl,"Z(Impedance X):%8.4f", ztem[id]);
  	  	text_len   = strlen(labl);
  	  	XDrawString(display,px_3d_map,gc,550,
			660,labl,text_len);
                refresh_3d();
                break;
            }
         }
   /* if ButtonPress occurs on the button window, do
      the specified command */
         for (i=0; i<9; i++) {
            if (event.xbutton.window == bt[i]) {
               button = event.xbutton.button;
               /* get the matching button release on the same button */
               while (1) {
                  while (XCheckTypedEvent(display,ButtonPress,&event));
                  XMaskEvent(display, ButtonReleaseMask, &event);
                  if (event.xbutton.button == button) {
                     paint_3d_b();
                     break;
                  }
               }
               switch (i) {
               case 0 : rotate(UP);  
                 plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
			yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
                 refresh_3d(); 
                 break;
               case 1 : rotate(DOWN);   
                 plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
			yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
                 refresh_3d(); 
		 break;
               case 2 : rotate(RIGHT); 
                 plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
			yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
                 refresh_3d(); 
                 break;
               case 3 : rotate(LEFT);
                 plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
			yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
                 refresh_3d(); 
                 break;
               case 4 : rotate(XOY);
                 plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
			yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
                 refresh_3d(); 
                 break;
               case 5 : rotate(YOZ); 
                 plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
			yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
                 refresh_3d(); 
                 break;
               case 6 : rotate(ZOX); 
                 plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
			yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
                 refresh_3d(); 
                 break;
	       case 7 : rotate(ANY); 
                 plot3d(idx1,idx2,xtem,ytem,ztem,xaray,
			yaray,maxpt,max_p,xs,ys,xd,yd,iflg,ics,icc);
                 refresh_3d(); 
                 break;
	       case 8 :  return; break;
               default: break;
               } 
            } /*end if */
         } /*end for */
         break;
    /*
      case ConfigureNotify:
         Width  = event.xconfigure.width;
         Height = event.xconfigure.height;
         break; */

      default:
         break;
    }
   }
}

/***********************************************************
 * draw frame and X,Y Z Title and Plot Title
 */
frame()
{
   char time[100];
   int  text_width, text_xpos, text_ypos, font_height, i, j;

/*
   XDrawRectangle(display,px_3d_map,gc,200,70,615,530);
*/
   XDrawRectangle(display,px_3d_map,gc,Xmin,Ymin,Xmax-Xmin,Ymax-Ymin);
   XCopyArea(display, hydro_pixmap, px_3d_map, gc,
             0, 0, 96, 48, 20,20);

   /* Find a font and load it */
   font_name = startup_.lgfont;
   if (!(font_info = XLoadQueryFont(display,font_name))){
      fprintf(stderr,"Could not open font %s\n",font_name);
      exit(1);
   }
   XSetFont(display, gc, font_info->fid);
#ifdef _LOGO_OH
   XDrawString(display,px_3d_map, gc, 80, 55,
                        "Ontario Hydro", 13);
#endif
#ifdef _LOGO_EPRI
   XDrawString(display,px_3d_map, gc, 10, 55,
                        "Leadership in Science and Technololgy", 37);
#endif

/*  text_width = lens_(ptit_.ftitle); */
    text_width = textlen(ptit_.ftitle);
    text_xpos = (X_DIM - XTextWidth(font_info,ptit_.ftitle,text_width))/2;
    text_ypos = Y_DIM - 35;
    XDrawString(display,px_3d_map, gc,text_xpos, text_ypos,
        ptit_.ftitle,text_width);

   font_name = startup_.defont;
   if (!(font_info = XLoadQueryFont(display,font_name))){
      fprintf(stderr,"Could not open font %s\n",font_name);
      exit(1);
   }
   XSetFont(display, gc, font_info->fid);

   XDrawString(display,px_3d_map, gc, Xmin, Ymax+20, "X : ", 4);
   XDrawString(display,px_3d_map, gc, Xmin, Ymax+40, "Y : ", 4);
   XDrawString(display,px_3d_map, gc, Xmin, Ymax+60, "Z : ", 4);
   XDrawString(display,px_3d_map, gc, Xmin+25, Ymax+20, xlab, lx);
   XDrawString(display,px_3d_map, gc, Xmin+25, Ymax+40, ylab, ly);
   XDrawString(display,px_3d_map, gc, Xmin+25, Ymax+60, zlab, lz);

   j = 6;

   while(j && strspn(ptit_.ptitle[--j], " "));

   j++;
   /* write current time */
   get_time(time);
   font_height = font_info->max_bounds.ascent + font_info->max_bounds.descent;
   text_width   = strlen(time);
   text_xpos =  Xmax - XTextWidth(font_info,time,text_width) - 10;
   text_ypos = Ymin - (j+8) * font_height / 2;
   XDrawString(display,px_3d_map, gc,text_xpos, text_ypos,time,text_width);

        text_width = 80;
        for(i=0; i<j; i++) {
            text_ypos = Ymin - (j-2*i+6)*font_height/2;

        ptit_.ptitle[i][79] = '\0';
        XDrawString(display,px_3d_map,gc,Xmin,text_ypos,ptit_.ptitle[i],

                text_width);
        }
}

/***********************************************************
 * draw labels on the buttons
 */
paint_3d_b()
{
   static char
 		 *label[]={"Increase Theta",
		          "Decrease Theta",
		          "Increase Phi",
			  "Decrease Phi",
			  "XOY Plane",
			  "YOZ Plane",
			  "ZOX Plane",
			  "Input View Point",
			  "Continue"};
   int i;

   for (i=0; i<9; i++){ 
       XSetWindowBackground(display,bt[i],background);
       XClearWindow(display,bt[i]);
       XDrawString(display,bt[i],bgcr,2,
		font_info->max_bounds.ascent,
        	label[i], strlen(label[i]));
   }
}

/***********************************************************
 * 3D plot parameter setting
 */
init_3d_data_(xm,xa,ym,ya,zm,za,xl,lenx,yl,leny,zl,lenz)
float *xm, *xa, *ym, *ya, *zm, *za;
#ifdef VMS
struct dsc$descriptor_s *xl, *yl, *zl;
#else
char *xl, *yl, *zl;
#endif
int *lenx, *leny, *lenz;
{
   lx= *lenx; ly= *leny; lz= *lenz;
#ifdef VMS
   strncpy(xlab, xl->dsc$a_pointer,lx);
   strncpy(ylab, yl->dsc$a_pointer,ly);
   strncpy(zlab, zl->dsc$a_pointer,lz);
#else
   strncpy(xlab,xl,lx);
   strncpy(ylab,yl,ly);
   strncpy(zlab,zl,lz);
#endif
   xlab[lx]='\0';
   ylab[ly]='\0';
   zlab[lz]='\0';
   xmin= *xm; xmax= *xa; ymin= *ym; ymax= *ya; zmin= *zm; zmax= *za;
   Identity(scal); 
   if ((xmax-xmin) > 1.0e-10) scal[0][0] = 1.0/(xmax-xmin);
   if ((ymax-ymin) > 1.0e-10) scal[1][1] = 1.0/(ymax-ymin);
   if ((zmax-zmin) > 1.0e-10) scal[2][2] = 1.0/(zmax-zmin);
   oo.x=xmin; oo.y=ymin; oo.z=zmin;
   xo.x=xmax; xo.y=ymin; xo.z=zmin;
   yo.x=xmin; yo.y=ymax; yo.z=zmin;
   zo.x=xmin; zo.y=ymin; zo.z=zmax;
   xoy.x=xmax; xoy.y=ymax; xoy.z=zmin;
   yoz.x=xmin; yoz.y=ymax; yoz.z=zmax;
   zox.x=xmax; zox.y=ymin; zox.z=zmax;
   xyz.x=xmax; xyz.y=ymax; xyz.z=zmax;
}

/***********************************************************
 * unmap and destroy the window and all subwindows;
 * close display connection
 */
x2end_()
{
   XUnloadFont(display, font_info->fid);
   XFreeGC(display, gc);
   XFreePixmap( display, px_3d_map );
   XDestroyWindow( display, wd );
   XFlush( display );
   XCloseDisplay(display);
}    /* x3d.c */
